@@ -93,7 +93,7 @@ static int dummy_printf(const char *__restrict __format, ...)
 #define INTEL_LPMD_SERVICE_INTERFACE	"org.freedesktop.intel_lpmd"
 
 typedef enum {
-	TERMINATE, LPM_FORCE_ON, LPM_FORCE_OFF, LPM_AUTO, SUV_MODE_ENTER, SUV_MODE_EXIT, HFI_EVENT,
+	TERMINATE, LPM_FORCE_ON, LPM_FORCE_OFF, LPM_AUTO, HFI_EVENT,
 } message_name_t;
 
 #define MAX_MSG_SIZE		512
@@ -109,6 +109,41 @@ typedef struct {
 #define MAX_STATE_NAME		32
 #define MAX_CONFIG_LEN		64
 
+enum lpmd_states {
+	LPMD_OFF,
+	LPMD_ON,
+	LPMD_AUTO,
+	LPMD_FREEZE,
+	LPMD_RESTORE,
+	LPMD_TERMINATE,
+};
+
+enum lpmd_update_reason {
+	UPDATE_USER,
+	UPDATE_UTIL,
+	UPDATE_HFI,
+	UPDATE_CPUHOTPLUG,
+	UPDATE_WLT,
+};
+
+typedef struct {
+	int util_cpu;	/* From Util monitor */
+	int util_sys;	/* From Util monitor */
+	int util_gfx;	/* From Util monitor */
+	int wlt_hint;	/* From WLT monitor */
+	int polling_interval;
+	int need_update;
+}lpmd_data_t;
+
+enum default_config_state {
+	DEFAULT_OFF,	/* lpmd force off: state with all default power settings */
+	DEFAULT_ON,	/* lpmd force on: state with global CPU/IRQ/ITMT/EPP configurations */
+	DEFAULT_HFI,	/* LPM state with CPU isolation based on HFI hints only */
+	CONFIG_STATE_BASE,
+	MAX_STATES = CONFIG_STATE_BASE + MAX_CONFIG_STATES,
+	STATE_NONE = MAX_STATES,
+};
+
 typedef struct {
 	int id;
 	int valid;
@@ -139,9 +174,16 @@ typedef struct {
 	int itmt_state;
 	int irq_migrate;
 
+	int balance_slider_ac;
+	int slider_offset_ac;
+	int balance_slider_dc;
+	int slider_offset_dc;
+
 	// Private state variables, not configurable
 	int entry_load_sys;
 	int entry_load_cpu;
+	int cpumask_idx;
+	int steady;
 }lpmd_config_state_t;
 
 // lpmd config data
@@ -151,11 +193,20 @@ typedef struct {
 	int balanced_def;
 	int powersaver_def;
 	int hfi_lpm_enable;
-	int hfi_suv_enable;
 	int wlt_hint_enable;
 	int wlt_hint_poll_enable;
 	int wlt_proxy_enable;
-	int util_enable;
+	int wlt_hint_mask;
+
+	union {
+		struct {
+			uint32_t util_sys_enable:1;
+			uint32_t util_cpu_enable:1;
+			uint32_t util_gfx_enable:1;
+			uint32_t util_reserved:29;
+		};
+		uint32_t util_enable;
+	};
 	int util_entry_threshold;
 	int util_exit_threshold;
 	int util_entry_delay;
@@ -170,7 +221,14 @@ typedef struct {
 	char cpu_config[MAX_CONFIG_LEN];
 	int config_state_count;
 	int tdp;
-	lpmd_config_state_t config_states[MAX_CONFIG_STATES];
+
+	int balance_slider_def_ac;
+	int slider_offset_def_ac;
+	int balance_slider_def_dc;
+	int slider_offset_def_dc;
+
+	lpmd_config_state_t config_states[MAX_STATES];
+	lpmd_data_t data;
 } lpmd_config_t;
 
 enum lpm_cpu_process_mode {
@@ -181,24 +239,17 @@ enum lpm_cpu_process_mode {
 	LPM_CPU_MODE_MAX = LPM_CPU_POWERCLAMP,
 };
 
-enum lpm_command {
-	USER_ENTER, /* Force enter LPM and always stay in LPM */
-	USER_AUTO, /* Allow oppotunistic LPM based on util/hfi request */
-	USER_EXIT, /* Force exit LPM and never enter LPM */
-	HFI_ENTER,
-	HFI_EXIT,
-	HFI_SUV_ENTER,
-	HFI_SUV_EXIT,
-	DBUS_SUV_ENTER,
-	DBUS_SUV_EXIT,
-	UTIL_ENTER,
-	UTIL_EXIT,
-	LPM_CMD_MAX,
-};
-
+#define NUM_USER_CPUMASKS	10
 enum cpumask_idx {
-	CPUMASK_LPM_DEFAULT, CPUMASK_ONLINE, CPUMASK_HFI, CPUMASK_HFI_BANNED, CPUMASK_HFI_SUV, /* HFI Survivability mode */
-	CPUMASK_HFI_LAST, CPUMASK_UTIL, CPUMASK_MAX,
+	CPUMASK_LPM_DEFAULT,
+	CPUMASK_ONLINE,
+	CPUMASK_HFI,
+	CPUMASK_HFI_BANNED,
+	CPUMASK_HFI_LAST,
+	CPUMASK_UTIL,
+	CPUMASK_USER,
+	CPUMASK_MAX = CPUMASK_USER + NUM_USER_CPUMASKS,
+	CPUMASK_NONE = CPUMASK_MAX,
 };
 
 #define UTIL_DELAY_MAX		5000
@@ -233,61 +284,27 @@ enum power_profile_daemon_mode {
 	PPD_INVALID
 };
 
-/* Helpers for entering LPMode */
-void set_lpm_epp(int val);
-int get_lpm_epp(void);
-void set_lpm_epb(int val);
-int get_lpm_epb(void);
-int get_epp_epb(int *epp, char *epp_string, int size, int *epb);
-void set_lpm_itmt(int val);
-int get_lpm_itmt(void);
-int get_itmt(void);
-int set_lpm_irq(cpu_set_t *cpumask, int action);
-int set_lpm_cpus(enum cpumask_idx idx);
-
 /* lpmd_main.c */
 int in_debug_mode(void);
 int do_platform_check(void);
 
 /* lpmd_proc.c: interfaces */
+lpmd_config_t *get_lpmd_config(void);
+
 int lpmd_lock(void);
 int lpmd_unlock(void);
-int in_lpm(void);
-int in_hfi_lpm(void);
-int in_suv_lpm(void);
-int in_auto_mode(void);
-int get_idle_percentage(void);
-int get_idle_duration(void);
-int get_cpu_mode(void);
-int has_hfi_lpm_monitor(void);
-int has_hfi_suv_monitor(void);
-int has_util_monitor(void);
-int get_util_entry_interval(void);
-int get_util_exit_interval(void);
-int get_util_entry_threshold(void);
-int get_util_exit_threshold(void);
-int get_util_entry_hyst(void);
-int get_util_exit_hyst(void);
-void set_ignore_itmt(void);
-
-int process_lpm(enum lpm_command cmd);
-int process_lpm_unlock(enum lpm_command cmd);
-int freeze_lpm(void);
-int restore_lpm(void);
 
 void lpmd_terminate(void);
 void lpmd_force_on(void);
 void lpmd_force_off(void);
 void lpmd_set_auto(void);
-void lpmd_suv_enter(void);
-void lpmd_suv_exit(void);
-void lpmd_notify_hfi_event(void);
 
 int is_on_battery(void);
 int get_ppd_mode(void);
 
 /* lpmd_proc.c: init func */
 int lpmd_main(void);
+void update_reason(int reason);
 
 /* lpmd_dbus_server.c */
 int intel_dbus_server_init(gboolean (*exit_handler)(void));
@@ -295,68 +312,96 @@ int intel_dbus_server_init(gboolean (*exit_handler)(void));
 /* lpmd_config.c */
 int lpmd_get_config(lpmd_config_t *lpmd_config);
 
-/* util.c */
-int periodic_util_update(lpmd_config_t *lpmd_config, int wlt_index);
-int util_init(lpmd_config_t *lpmd_config);
-int use_config_states(void);
-void reset_config_state(void);
+/* lpmd_state_machine.c */
+int update_lpmd_state(int state);
+int get_lpmd_state(void);
+int lpmd_init_config_state(lpmd_config_state_t *state);
+int lpmd_build_config_states(lpmd_config_t *config);
+int lpmd_enter_next_state(void);
 
-/* cpu.c */
-int check_cpu_capability(lpmd_config_t *lpmd_config);
-int init_cpu(char *cmd_cpus, enum lpm_cpu_process_mode mode, int lp_mode_epp);
-int process_cpus(int enter, enum lpm_cpu_process_mode mode);
-int parse_cpu_str(char *buf, enum cpumask_idx idx);
+/* lpmd_util.c */
+int util_update(lpmd_config_t *lpmd_config);
 
-int is_cpu_lcore(int cpu);
-int is_cpu_ecore(int cpu);
-int is_cpu_pcore(int cpu);
-/* cpu.c: helpers */
-int is_cpu_online(int cpu);
-int is_cpu_for_lpm(int cpu);
-int get_max_cpus(void);
-int get_max_online_cpu(void);
+/* lpmd_hfi.c */
+int hfi_init(void);
+int hfi_kill(void);
+int hfi_update(void);
 
-char* get_cpus_str(enum cpumask_idx idx);
-char* get_lpm_cpus_hexstr(void);
-int has_lpm_cpus(void);
-int has_cpus(enum cpumask_idx idx);
+/* lpmd_wlt.c */
+int wlt_init(void);
+int wlt_exit(void);
+int wlt_update(int fd);
 
-void copy_cpu_mask_exclude(enum cpumask_idx source, enum cpumask_idx dest, enum cpumask_idx exlude);
-void copy_cpu_mask(enum cpumask_idx source, enum cpumask_idx dest);
-void copy_cpu_mask_exclude(enum cpumask_idx source, enum cpumask_idx dest, enum cpumask_idx exlude);
+/* lpmd_misc.c */
+int itmt_init(void);
+int get_itmt(void);
+int process_itmt(lpmd_config_state_t *state);
 
-cpu_set_t *get_cpumask(enum cpumask_idx source);
-int cpumask_to_str(cpu_set_t *cpumask, char *buf, int size);
-int cpumask_to_hexstr(cpu_set_t *cpumask, char *buf, int size);
-int cpumask_to_str_reverse(cpu_set_t *mask, char *buf, int size);
+int epp_epb_init(void);
+int get_epp_epb(int *epp, char *epp_str, int size, int *epb);
+int process_epp_epb(lpmd_config_state_t *state);
 
-int is_equal(enum cpumask_idx idx1, enum cpumask_idx idx2);
+void process_balance_slider_default_update(lpmd_config_t *config);
+void process_slider_offset_default_update(lpmd_config_t *config);
+void process_slider(lpmd_config_t *config, lpmd_config_state_t *state);
 
-int add_cpu(int cpu, enum cpumask_idx idx);
-void reset_cpus(enum cpumask_idx idx);
-int set_lpm_cpus(enum cpumask_idx new);
+/* lpmd_irq.c */
+int irq_init(void);
+int process_irq(lpmd_config_state_t *state);
+
+/* lpmd_cgroup.c*/
+int cgroup_init(lpmd_config_t *config);
+int cgroup_cleanup(void);
+int process_cgroup(lpmd_config_state_t *state, enum lpm_cpu_process_mode mode);
+
+/* lpmd_uevent.c */
 int uevent_init(void);
 int check_cpu_hotplug(void);
 
-/* cpu.c : APIs for SUV mode support */
-int process_suv_mode(enum lpm_command cmd);
-int has_suv_support(void);
+/* lpmd_cpu.c */
+int detect_supported_platform(lpmd_config_t *lpmd_config);
+int detect_cpu_topo(lpmd_config_t *lpmd_config);
+int detect_lpm_cpus(char *cmd_cpus);
 
-/* irq.c */
-int init_irq(void);
-int process_irqs(int enter, enum lpm_cpu_process_mode mode);
-int update_lpm_irq(cpu_set_t *cpumask, int action);
+int is_cpu_ecore(int cpu);
+int is_cpu_pcore(int cpu);
 
-/* hfi.c */
-int hfi_init(void);
-int hfi_kill(void);
-void hfi_receive(void);
+/* lpmd_cpumask.c */
+int is_cpu_online(int cpu);
+int get_max_cpus(void);
+void set_max_cpus(int num);
+int get_max_online_cpu(void);
+void set_max_online_cpu(int num);
+int cpu_migrate(int cpu);
+int cpu_clear_affinity(void);
+
+int cpumask_alloc(void);
+int cpumask_free(enum cpumask_idx idx);
+int cpumask_reset(enum cpumask_idx idx);
+
+int cpumask_add_cpu(int cpu, enum cpumask_idx idx);
+int cpumask_init_cpus(char *buf, enum cpumask_idx idx);
+int cpumask_nr_cpus(enum cpumask_idx idx);
+int cpumask_has_cpu(enum cpumask_idx idx);
+
+int cpumask_equal(enum cpumask_idx idx1, enum cpumask_idx idx2);
+void cpumask_copy(enum cpumask_idx source, enum cpumask_idx dest);
+void cpumask_exclude_copy(enum cpumask_idx source, enum cpumask_idx dest, enum cpumask_idx exlude);
+
+char *get_cpus_str(enum cpumask_idx idx);
+char* get_cpus_hexstr(enum cpumask_idx idx);
+char *get_proc_irq_str(enum cpumask_idx idx);
+char *get_irqbalance_str(enum cpumask_idx idx);
+char *get_cpu_isolation_str(enum cpumask_idx idx);
+uint8_t *get_cgroup_systemd_vals(enum cpumask_idx idx, int *size);
 
 /* socket.c */
 int socket_init_connection(char *name);
 int socket_send_cmd(char *name, char *data);
 
 /* helper */
+int copy_user_string(char *src, char *dst, int size);
+int lpmd_read_str(char *path, char *str, int size);
 int lpmd_write_str(const char *name, char *str, int print_level);
 int lpmd_write_str_verbose(const char *name, char *str, int print_level);
 int lpmd_write_str_append(const char *name, char *str, int print_level);
