@@ -25,111 +25,17 @@
 #define CONFIG_FILE_NAME "intel_lpmd_config.xml"
 #define MAX_FILE_NAME_PATH	128
 
-static void lpmd_dump_config(lpmd_config_t *lpmd_config)
-{
-	int i;
-	lpmd_config_state_t *state;
-
-	if (!lpmd_config)
-		return;
-
-	lpmd_log_info ("Mode:%d\n", lpmd_config->mode);
-	lpmd_log_info ("HFI LPM Enable:%d\n", lpmd_config->hfi_lpm_enable);
-	lpmd_log_info ("HFI SUV Enable:%d\n", lpmd_config->hfi_suv_enable);
-	lpmd_log_info ("WLT Hint Enable:%d\n", lpmd_config->wlt_hint_enable);
-	lpmd_log_info ("WLT Proxy Enable:%d\n", lpmd_config->wlt_proxy_enable);
-	lpmd_log_info ("WLT Proxy Enable:%d\n", lpmd_config->wlt_hint_poll_enable);
-	lpmd_log_info ("Util entry threshold:%d\n", lpmd_config->util_entry_threshold);
-	lpmd_log_info ("Util exit threshold:%d\n", lpmd_config->util_exit_threshold);
-	lpmd_log_info ("Util LP Mode CPUs:%s\n", lpmd_config->lp_mode_cpus);
-	lpmd_log_info ("EPP in LP Mode:%d\n", lpmd_config->lp_mode_epp);
-
-	if (!lpmd_config->config_state_count)
-		return;
-
-	lpmd_log_info ("CPU Family:%d\n", lpmd_config->cpu_family);
-	lpmd_log_info ("CPU Model:%d\n", lpmd_config->cpu_model);
-	lpmd_log_info ("CPU Config:%s\n", lpmd_config->cpu_config);
-
-	for (i = 0; i < lpmd_config->config_state_count; ++i) {
-		state = &lpmd_config->config_states[i];
-
-		lpmd_log_info ("ID:%d\n", state->id);
-		lpmd_log_info ("\tName:%s\n", state->name);
-		lpmd_log_info ("\tentry_system_load_thres:%d\n", state->entry_system_load_thres);
-		lpmd_log_info ("\texit_system_load_thres:%d\n", state->exit_system_load_thres);
-		lpmd_log_info ("\texit_system_load_hyst:%d\n", state->exit_system_load_hyst);
-		lpmd_log_info ("\tentry_cpu_load_thres:%d\n", state->enter_cpu_load_thres);
-		lpmd_log_info ("\texit_cpu_load_thres:%d\n", state->exit_cpu_load_thres);
-		lpmd_log_info ("\tentry_gfx_load_thres:%d\n", state->enter_gfx_load_thres);
-		lpmd_log_info ("\texit_gfx_load_thres:%d\n", state->exit_gfx_load_thres);
-		lpmd_log_info ("\tWLT Type:%d\n", state->wlt_type);
-		lpmd_log_info ("\tmin_poll_interval:%d\n", state->min_poll_interval);
-		lpmd_log_info ("\tmax_poll_interval:%d\n", state->max_poll_interval);
-		lpmd_log_info ("\tpoll_interval_increment:%d\n", state->poll_interval_increment);
-		lpmd_log_info ("\tEPP:%d\n", state->epp);
-		lpmd_log_info ("\tEPB:%d\n", state->epb);
-		lpmd_log_info ("\tITMTState:%d\n", state->itmt_state);
-		lpmd_log_info ("\tIRQMigrate:%d\n", state->irq_migrate);
-		if (state->active_cpus[0] != '\0')
-			lpmd_log_info ("\tactive_cpus:%s\n", state->active_cpus);
-		lpmd_log_info ("\tisland_0_number_p_cores:%d\n", state->island_0_number_p_cores);
-		lpmd_log_info ("\tisland_0_number_e_cores:%d\n", state->island_0_number_e_cores);
-		lpmd_log_info ("\tisland_1_number_p_cores:%d\n", state->island_1_number_p_cores);
-		lpmd_log_info ("\tisland_1_number_e_cores:%d\n", state->island_1_number_e_cores);
-		lpmd_log_info ("\tisland_2_number_p_cores:%d\n", state->island_2_number_p_cores);
-		lpmd_log_info ("\tisland_2_number_e_cores:%d\n", state->island_2_number_e_cores);
-	}
-}
-
-/* Set all of them Some of the operations are redundant, but it is useful to*/
-static void lpmd_init_config_state(lpmd_config_state_t *state)
-{
-	state->id = -1;
-	state->valid = 0;
-	state->name[0] = '\0';
-
-	state->wlt_type = -1;
-
-	state->entry_system_load_thres = 0;
-	state->exit_system_load_thres = 0;
-	state->exit_system_load_hyst = 0;
-	state->enter_cpu_load_thres = 0;
-	state->exit_cpu_load_thres = 0;
-	state->enter_gfx_load_thres = 0;
-	state->exit_gfx_load_thres = 0;
-
-	state->min_poll_interval = 0;
-	state->max_poll_interval = 0;
-	state->poll_interval_increment = 0;
-
-	state->epp = SETTING_IGNORE;
-	state->epb = SETTING_IGNORE;
-	state->active_cpus[0] = '\0';
-
-	state->island_0_number_p_cores = 0;
-	state->island_0_number_e_cores = 0;
-	state->island_1_number_p_cores = 0;
-	state->island_1_number_e_cores = 0;
-	state->island_2_number_p_cores = 0;
-	state->island_2_number_e_cores = 0;
-
-	state->itmt_state = SETTING_IGNORE;
-	state->irq_migrate = SETTING_IGNORE;
-
-	state->entry_load_sys = 0;
-	state->entry_load_cpu = 0;
-}
-
-static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_state_t *state)
+static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *config, int idx)
 {
 	xmlNode *cur_node = NULL;
 	char *tmp_value;
 	char *pos;
+	lpmd_config_state_t *state = &config->config_states[idx];
 
 	if (!doc || !a_node || !state)
 		return;
 
+	lpmd_log_debug("lpmd_parse_state: %d\n", idx);
 	lpmd_init_config_state(state);
 
 	for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
@@ -188,35 +94,31 @@ static void lpmd_parse_state(xmlDoc *doc, xmlNode *a_node, lpmd_config_state_t *
 					if (!strncmp (tmp_value, "-1", strlen ("-1")))
 						state->active_cpus[0] = '\0';
 					else
-						snprintf (state->active_cpus, sizeof(state->active_cpus), "%s", tmp_value);
+						copy_user_string(tmp_value, state->active_cpus, sizeof(state->active_cpus));
 				}
+				if (!strncmp((const char*)cur_node->name, "BalanceSliderAC", strlen("BalanceSliderAC")))
+					state->balance_slider_ac = strtol (tmp_value, &pos, 10);
+				if (!strncmp((const char*)cur_node->name, "SliderOffsetAC", strlen("SliderOffsetAC")))
+					state->slider_offset_ac = strtol (tmp_value, &pos, 10);
+
+				if (!strncmp((const char*)cur_node->name, "BalanceSliderDC", strlen("BalanceSliderDC")))
+					state->balance_slider_dc = strtol (tmp_value, &pos, 10);
+				if (!strncmp((const char*)cur_node->name, "SliderOffsetDC", strlen("SliderOffsetDC")))
+					state->slider_offset_dc = strtol (tmp_value, &pos, 10);
+
 				xmlFree(tmp_value);
 			}
 		}
 	}
 }
 
-static int validate_config_state(lpmd_config_t *lpmd_config, lpmd_config_state_t *state)
-{
-	if (!state->enter_gfx_load_thres && (lpmd_config->wlt_hint_enable || lpmd_config->wlt_proxy_enable)) {
-		if (state->wlt_type >=0 && state->wlt_type < WLT_INVALID)
-			state->valid = 1;
-	} else {
-		if ((state->enter_cpu_load_thres > 0 && state->enter_cpu_load_thres <= 100) ||
-		    (state->entry_system_load_thres > 0 && state->entry_system_load_thres <= 100) ||
-		    (state->enter_gfx_load_thres > 0 && state->enter_gfx_load_thres <= 100))
-			state->valid = 1;
-	}
-	return 0;
-}
-
 static int is_wildcard(char *str)
 {
 	if (!str)
 		return 1;
-	if (strncmp(str, "*", strlen("*")))
+	if (!strncmp(str, "*", strlen("*")))
 		return 1;
-	if (strncmp(str, " * ", strlen(" * ")))
+	if (!strncmp(str, " * ", strlen(" * ")))
 		return 1;
 
 	return 0;
@@ -282,15 +184,30 @@ static void lpmd_parse_states(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_
 
 				if (lpmd_config->config_state_count >= MAX_CONFIG_STATES)
 					break;
-				lpmd_parse_state (doc, cur_node->children, &lpmd_config->config_states[config_state_count]);
-				validate_config_state(lpmd_config, &lpmd_config->config_states[config_state_count]);
-				config_state_count += lpmd_config->config_states[config_state_count].valid;
+				lpmd_parse_state (doc, cur_node->children, lpmd_config, CONFIG_STATE_BASE + config_state_count);
+				config_state_count ++;
 			}
 		}
 	}
+	lpmd_log_debug("Found %d config states\n", config_state_count);
 	lpmd_config->config_state_count = config_state_count;
 }
 
+static void lpmd_init_config(lpmd_config_t *config)
+{
+	config->performance_def = config->balanced_def = config->powersaver_def = LPM_FORCE_OFF;
+	config->lp_mode_epp = -1;
+	config->data.util_sys = -1;
+	config->data.util_cpu = -1;
+	config->data.util_gfx = -1;
+	config->data.wlt_hint = -1;
+	config->balance_slider_def_ac = -1;
+	config->balance_slider_def_dc = -1;
+	config->slider_offset_def_ac = -1;
+	config->slider_offset_def_dc = -1;
+	config->wlt_hint_mask = -1;
+}
+
 static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_config)
 {
 	xmlNode *cur_node = NULL;
@@ -300,8 +217,8 @@ static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_co
 	if (!doc || !a_node || !lpmd_config)
 		return LPMD_ERROR;
 
-	lpmd_config->performance_def = lpmd_config->balanced_def = lpmd_config->powersaver_def = LPM_FORCE_OFF;
-	lpmd_config->lp_mode_epp = -1;
+	lpmd_init_config(lpmd_config);
+
 	for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
 		if (cur_node->type == XML_ELEMENT_NODE) {
 			tmp_value = (char*) xmlNodeListGetString (doc, cur_node->xmlChildrenNode, 1);
@@ -320,13 +237,6 @@ static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_co
 							|| (lpmd_config->hfi_lpm_enable != 1 && lpmd_config->hfi_lpm_enable != 0))
 						goto err;
 				}
-				else if (!strncmp((const char*)cur_node->name, "HfiSuvEnable", strlen("HfiSuvEnable"))) {
-					errno = 0;
-					lpmd_config->hfi_suv_enable = strtol (tmp_value, &pos, 10);
-					if (errno || *pos != '\0'
-							|| (lpmd_config->hfi_suv_enable != 1 && lpmd_config->hfi_suv_enable != 0))
-						goto err;
-				}
 				else if (!strncmp((const char*)cur_node->name, "WLTHintEnable", strlen("WLtHintEnable"))) {
 					errno = 0;
 					lpmd_config->wlt_hint_enable = strtol (tmp_value, &pos, 10);
@@ -334,6 +244,10 @@ static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_co
 							|| (lpmd_config->wlt_hint_enable != 1 && lpmd_config->wlt_hint_enable != 0))
 						goto err;
 				}
+				else if (!strncmp((const char*)cur_node->name, "WLTHintMask", strlen("WLTHintMask"))) {
+					errno = 0;
+					lpmd_config->wlt_hint_mask = strtol (tmp_value, &pos, 10);
+				}
 				else if (!strncmp((const char*)cur_node->name, "WLTHintPollEnable", strlen("WLtHintPollEnable"))) {
 					errno = 0;
 					lpmd_config->wlt_hint_poll_enable = strtol (tmp_value, &pos, 10);
@@ -418,8 +332,7 @@ static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_co
 					if (!strncmp (tmp_value, "-1", strlen ("-1")))
 						lpmd_config->lp_mode_cpus[0] = '\0';
 					else
-						snprintf (lpmd_config->lp_mode_cpus, sizeof(lpmd_config->lp_mode_cpus),
-									"%s", tmp_value);
+						copy_user_string(tmp_value, lpmd_config->lp_mode_cpus, sizeof(lpmd_config->lp_mode_cpus));
 				}
 				else if (!strncmp((const char*)cur_node->name, "PerformanceDef", strlen ("PerformanceDef"))) {
 					errno = 0;
@@ -467,9 +380,30 @@ static int lpmd_fill_config(xmlDoc *doc, xmlNode *a_node, lpmd_config_t *lpmd_co
 					errno = 0;
 					lpmd_parse_states(doc, cur_node->children, lpmd_config);
 				}
+				else if (!strncmp((const char*)cur_node->name, "BalancedSliderAC", strlen ("BalancedSliderAC"))) {
+					errno = 0;
+					lpmd_config->balance_slider_def_ac = strtol (tmp_value, &pos, 10);
+
+				}
+				else if (!strncmp((const char*)cur_node->name, "BalancedSliderDC", strlen ("BalancedSliderDC"))) {
+					errno = 0;
+					lpmd_config->balance_slider_def_dc = strtol (tmp_value, &pos, 10);
+				}
+				else if (!strncmp((const char*)cur_node->name, "SliderOffsetAC", strlen ("SliderOffsetAC"))) {
+					errno = 0;
+					lpmd_config->slider_offset_def_ac = strtol (tmp_value, &pos, 10);
+				}
+				else if (!strncmp((const char*)cur_node->name, "SliderOffsetDC", strlen ("SliderOffsetDC"))) {
+					errno = 0;
+					lpmd_config->slider_offset_def_dc = strtol (tmp_value, &pos, 10);
+				}
 				else {
-					lpmd_log_info ("Invalid configuration data\n");
-					goto err;
+					if (!strncmp((const char*)cur_node->name, "HfiSuvEnable", strlen("HfiSuvEnable"))) {
+						lpmd_log_debug("Ignore deprecated HfiSuvEnable setting\n");
+					} else {
+						lpmd_log_info ("Invalid configuration data\n");
+						goto err;
+					}
 				}
 				xmlFree (tmp_value);
 				continue;
@@ -481,12 +415,6 @@ err:			xmlFree (tmp_value);
 		}
 	}
 
-	/* use entry_threshold == 0 or exit_threshold == 0 to effectively disable util monitor */
-	if (lpmd_config->util_entry_threshold && lpmd_config->util_exit_threshold)
-		lpmd_config->util_enable = 1;
-	else
-		lpmd_config->util_enable = 0;
-
 	return LPMD_SUCCESS;
 }
 
@@ -553,7 +481,5 @@ process_xml:
 
 	xmlFreeDoc (doc);
 
-	lpmd_dump_config (lpmd_config);
-
 	return LPMD_SUCCESS;
 }
