@@ -58,6 +58,16 @@ static gboolean use_syslog;
 // Disable dbus
 static gboolean dbus_enable;
 
+static gboolean ignore_platform_check = FALSE;
+
+int do_platform_check(void)
+{
+	if (ignore_platform_check)
+		return 0;
+
+	return 1;
+}
+
 static GMainLoop *g_main_loop;
 
 #ifdef GDBUS
@@ -180,6 +190,7 @@ int main(int argc, char *argv[])
 			  { "loglevel=info", 0, 0, G_OPTION_ARG_NONE, &log_info, N_ ("Log severity: info level and up"), NULL },
 			  { "loglevel=debug", 0, 0, G_OPTION_ARG_NONE, &log_debug, N_ ("Log severity: debug level and up: Max logging"), NULL },
 			  { "dbus-enable", 0, 0, G_OPTION_ARG_NONE, &dbus_enable, N_ ( "Enable Dbus"), NULL },
+			  { "ignore-platform-check", 0, 0, G_OPTION_ARG_NONE, &ignore_platform_check, N_ ( "Ignore platform check"), NULL },
 			  { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL } };
 
 	if (!g_module_supported ()) {
@@ -202,9 +213,9 @@ int main(int argc, char *argv[])
 	g_option_context_add_main_entries (opt_ctx, options, NULL);
 
 	g_option_context_set_summary (opt_ctx,
-				"Intel Low Power Daemon based on system usage takes action "
-				"to reduce active power of the system.\n\n"
-				"Copyright (c) 2023, Intel Corporation\n"
+				"Intel Energy Optimizer (LPMD) Daemon based on system usage takes action "
+				"to improve energy efficiency the system.\n\n"
+				"Copyright (c) 2024, Intel Corporation\n"
 				"This program comes with ABSOLUTELY NO WARRANTY.\n"
 				"This work is licensed under GPL v2.\n\n"
 				"Use \"man intel_lpmd\" to get more details.");
