@@ -268,14 +268,6 @@ static unsigned int read_mperf_config(void) {
     return read_perf_counter_info_n(path, format);
 }
 
-/*helper - pperf reading */
-static unsigned int read_tsc_config(void) {
-    const char *const path = "/sys/bus/event_source/devices/msr/events/tsc";
-    const char *const format = "event=%x";
-
-    return read_perf_counter_info_n(path, format);
-}
-
 /*helper - pperf reading */
 static unsigned int read_msr_type(void) {
     const char *const path = "/sys/bus/event_source/devices/msr/type";
@@ -381,10 +373,9 @@ static int read_aperf_mperf_tsc_perf(struct thread_data *t, int cpu) {
 /*Calc perf [cpu utilization per core] difference from MSR registers */
 int update_perf_diffs(float *sum_norm_perf, int stat_init_only) {
 
-    int fd, maxed_cpu = -1;
+    int maxed_cpu = -1;
     float min_load = 100.0, min_s0 = 1.0, next_s0 = 1.0;
     float max_load = 0, max_2nd_load = 0, max_3rd_load = 0, next_load = 0;
-    uint64_t aperf_raw, mperf_raw, pperf_raw, tsc_raw, poll_cpu_us = 0;
     int t, min_s0_cpu = 0, first_pass = 1;
 
     for (t = 0; t < get_max_online_cpu(); t++) {
