@@ -22,9 +22,7 @@ do
 		echo "2 : LPM force on"
 		echo "3 : LPM force off"
 		echo "4 : LPM auto"
-		echo "5 : SUV_MODE Enter"
-		echo "6 : SUV_MODE Exit"
-		echo "7 : Quit"
+		echo "5 : Quit"
 	        echo -n " Enter choice: "
 	        read opt_no
 	fi
@@ -54,18 +52,10 @@ do
 		dbus-send --system --dest=org.freedesktop.intel_lpmd --print-reply /org/freedesktop/intel_lpmd org.freedesktop.intel_lpmd.LPM_AUTO
 		;;
 	5)
-		echo "5 : SUV_MODE Enter"
-		dbus-send --system --dest=org.freedesktop.intel_lpmd --print-reply /org/freedesktop/intel_lpmd org.freedesktop.intel_lpmd.SUV_MODE_ENTER
-		;;
-	6)
-		echo "6 : SUV_MODE Exit"
-		dbus-send --system --dest=org.freedesktop.intel_lpmd --print-reply /org/freedesktop/intel_lpmd org.freedesktop.intel_lpmd.SUV_MODE_EXIT
-		;;
-	7)
 		exit 0
 		;;
 	*)
-		echo "7 : Quit"
+		echo "5 : Quit"
 		echo "Invalid option"
 	esac
 	[ ! -z "$1" ] && break
